#!/bin/bash
#
# This script checks some startup conditions for PostgreSQL and prints warning
# and messages if they are violated.
#
# This script is executed with root privileges (called from PostgreSQL's init
# script). Please add checks for which the privileges of the "postgres" user
# are sufficient to postgresql-startup.
#
# Note: Checking kernel variables requires root privileges on security-enhanced
# kernels like grsecurity.

if [ -r /etc/postgresql/postmaster.conf ]; then
    source /etc/postgresql/postmaster.conf
fi

#
# Check that we have enough shared memory
#

# Get the current PostgreSQL buffers setting
BUFFERS="`grep "^ *shared_buffers" /etc/postgresql/postgresql.conf | cut -f2 -d= | cut -f1 -d# `" || true
[ "$BUFFERS" ] || BUFFERS=1000 # default value

# Each buffer is 8192 bytes, and we want bytes... plus some extra
SHMREQ=$(( BUFFERS * 8400 + 6000000 ))
SHMMAX="`cat /proc/sys/kernel/shmmax`"
if [ "$SHMMAX" -lt "$SHMREQ" ]
then
cat <<EOF >&2

/etc/postgresql/postgresql.conf sets shared_buffers=$BUFFERS.
Your kernel currently limits a shared memory segment to $SHMMAX bytes,
but PostgreSQL needs $SHMREQ bytes.

The server might not start due to this, without giving a proper error 
message. Please enlarge the amount of shared memory by issuing the 
following command (as root):

echo $SHMREQ > /proc/sys/kernel/shmmax
EOF
fi

#
# check that we can open enough files simultaneously
#

fmaxfile=/proc/sys/kernel/file-max
if [ ! -r $fmaxfile ]; then
    fmaxfile=/proc/sys/kernel/fs/file-max
    if [ ! -r $fmaxfile ]; then
        fmaxfile=/proc/sys/fs/file-max
        if [ ! -r $fmaxfile ]; then
            unset fmaxfile
        fi
    fi
fi

if [ -n "$fmaxfile" ]
then
    fmax=`cat $fmaxfile` || true
    if [ "$fmax" -lt ${KERNEL_FILE_MAX:=1032} ]
    then
        cat <<EOF >&2

/etc/postgresql/postmaster.conf sets KERNEL_FILE_MAX=$KERNEL_FILE_MAX,
but your kernel currently limits the number of simultaneously opened files
to $fmax.

The server might not start due to this, without giving a proper error 
message. Please enlarge the number of open files by issuing the 
following command (as root):

echo $KERNEL_FILE_MAX > $fmaxfile
EOF
    fi
fi

